@@ -59,7 +59,6 @@
 #include <shell/session.h>
 #include <interfaces/ilauncher.h>
 #include <interfaces/iproject.h>
-#include <interfaces/foregroundlock.h>
 
 #include "kdevideextension.h"
 
@@ -76,28 +75,22 @@ public:
     explicit KDevelopApplication(bool GUIenabled = true): KApplication(GUIenabled) {}
 
     void saveState( QSessionManager& sm ) {
-        if (KDevelop::Core::self() && KDevelop::Core::self()->sessionController())
-            sm.setRestartCommand(QStringList() << "kdevelop" << "-s" << KDevelop::Core::self()->sessionController()->activeSession()->id().toString());
+        if (KDevelop::Core::self() && KDevelop::Core::self()->sessionController()) {
+            QString x11SessionId = QString("%1_%2").arg(sm.sessionId()).arg(sm.sessionKey());
+            QString kdevelopSessionId = KDevelop::Core::self()->sessionController()->activeSession()->id().toString();
+
+            sm.setRestartCommand(QStringList() << QCoreApplication::applicationFilePath() << "-session" << x11SessionId << "-s" << kdevelopSessionId);
+        }
+
         KApplication::saveState(sm);
     }
-
-    virtual bool notify(QObject* receiver, QEvent* event);
 };
 
-bool KDevelopApplication::notify(QObject* receiver, QEvent* event)
-{
-    KDevelop::ForegroundLock lock(QThread::currentThread() == qApp->thread());
-    return KApplication::notify(receiver, event);
-}
-
 int main( int argc, char *argv[] )
 {
-    KDevelop::ForegroundLock lock;
-    
     static const char description[] = I18N_NOOP( "The KDevelop Integrated Development Environment" );
-    KAboutData aboutData( "kdevelop", 0, ki18n( "KDevelop" ),
-                          i18n("%1", QString(VERSION) ).toUtf8(), ki18n(description), KAboutData::License_GPL,
-                          ki18n( "Copyright 1999-2010, The KDevelop developers" ), KLocalizedString(), "http://www.kdevelop.org/" );
+    KAboutData aboutData( "kdevelop", 0, ki18n( "KDevelop" ), QByteArray(VERSION), ki18n(description), KAboutData::License_GPL,
+                          ki18n( "Copyright 1999-2011, The KDevelop developers" ), KLocalizedString(), "http://www.kdevelop.org/" );
     aboutData.addAuthor( ki18n("Andreas Pakulat"), ki18n( "Architecture, VCS Support, Project Management Support, QMake Projectmanager" ), "apaku@gmx.de" );
     aboutData.addAuthor( ki18n("Alexander Dymo"), ki18n( "Architecture, Sublime UI, Ruby support" ), "adymo@kdevelop.org" );
     aboutData.addAuthor( ki18n("David Nolden"), ki18n( "Definition-Use Chain, C++ Support, Code Navigation, Code Completion, Coding Assistance, Refactoring" ), "david.nolden.kdevelop@art-master.de" );
@@ -199,6 +192,8 @@ int main( int argc, char *argv[] )
                        "The binary that should be debugged must follow - including arguments.\n"
                        "Example: kdevelop --debug gdb myapp --foo bar"));
 
+    options.add("pid");
+           
     options.add("+files", ki18n( "Files to load" ));
 
     options.add(":", ki18n("Deprecated options:"));
@@ -237,7 +232,8 @@ int main( int argc, char *argv[] )
     
     if(args->isSet("ps"))
     {
-        session = KDevelop::SessionController::showSessionChooserDialog();
+        bool onlyRunning = args->isSet("pid");
+        session = KDevelop::SessionController::showSessionChooserDialog(QString(), onlyRunning);
         if(session.isEmpty())
             return 1;
     }
@@ -292,6 +288,22 @@ int main( int argc, char *argv[] )
 
     forever {
         KDevelop::SessionController::LockSessionState state = KDevelop::SessionController::tryLockSession(sessionId);
+        
+        if(args->isSet("pid"))
+        {
+            if(state.success)
+            {
+                kError() << session << sessionId << "is not running";
+                return 5;
+            }
+            if(args->isSet("pid"))
+            {
+                // Print the PID and we're ready
+                std::cout << state.holderPid << std::endl;
+                return 0;
+            }
+        }
+        
         if(!state) {
             QDBusInterface interface(QString("org.kdevelop.kdevelop-%1").arg(state.holderPid),
                                      "/kdevelop/MainWindow", "org.kdevelop.MainWindow",
@@ -464,6 +476,5 @@ int main( int argc, char *argv[] )
         args->clear();
     }
 
-    lock.unlock();
     return app.exec();
 }
