@@ -5,7 +5,7 @@
 int main(int argc, char** argv)
 {
     QApplication app(argc, argv);
-    %{APPNAME} foo;
+    %{APPNAMEID} foo;
     foo.show();
     return app.exec();
 }
