@@ -61,8 +61,6 @@
 #include <interfaces/iprojectcontroller.h>
 #include <interfaces/iselectioncontroller.h>
 
-Q_DECLARE_METATYPE(ProjectBaseItem*)
-
 using namespace KDevelop;
 
 // #define WARN_BEFORE_REFACTORING
@@ -243,7 +241,7 @@ void SimpleRefactoring::executeMoveIntoSourceAction() {
       
       KDevelop::IDocument* doc = ICore::self()->documentController()->documentForUrl(decl->url().toUrl());
       if(doc && doc->textDocument()) {
-        QString body = doc->textDocument()->text(decl->internalContext()->range().textRange());
+        QString body = doc->textDocument()->text(decl->internalContext()->rangeInCurrentRevision().textRange());
         SourceCodeInsertion ins(targetTopContext);
         QualifiedIdentifier namespaceIdentifier = decl->internalContext()->parentContext()->scopeIdentifier(false);
 
@@ -268,14 +266,14 @@ void SimpleRefactoring::executeMoveIntoSourceAction() {
           KMessageBox::error(ICore::self()->uiController()->activeMainWindow(), i18n("Insertion failed"));
           return;
         }
-        
+        KTextEditor::Range range = decl->internalContext()->rangeInCurrentRevision().textRange();
         lock.unlock();
         if(!ins.changes().applyAllChanges()) {
           KMessageBox::error(ICore::self()->uiController()->activeMainWindow(), i18n("Applying changes failed"));
           return;
         }
         
-        doc->textDocument()->replaceText(decl->internalContext()->range().textRange(), QString(";"));
+        doc->textDocument()->replaceText(range, QString(";"));
         ICore::self()->languageController()->backgroundParser()->addDocument(url.toUrl());
         ICore::self()->languageController()->backgroundParser()->addDocument(targetUrl);
         
@@ -337,7 +335,7 @@ DocumentChangeSet::ChangeResult applyChangesToDeclarations(QString oldName, QStr
 //         oldName = destructorForName(Identifier(oldName)).identifier().str();
       }
     }
-    DocumentChangeSet::ChangeResult result = changes.addChange(DocumentChange(top->url(), decl.data()->range(), oldName, newName));
+    DocumentChangeSet::ChangeResult result = changes.addChange(DocumentChange(top->url(), decl.data()->rangeInCurrentRevision(), oldName, newName));
     if(!result)
       return result;
   }
@@ -356,7 +354,7 @@ DocumentChangeSet::ChangeResult applyChanges(QString oldName, QString newName, D
        kDebug() << "found empty use";
        continue;
      }
-     DocumentChangeSet::ChangeResult result = changes.addChange(DocumentChange(context->url(), use.m_range, oldName, newName));
+     DocumentChangeSet::ChangeResult result = changes.addChange(DocumentChange(context->url(), context->transformFromLocalRevision(use.m_range), oldName, newName));
      if(!result)
        return result;
    }
@@ -388,7 +386,13 @@ void SimpleRefactoring::startInteractiveRename(KDevelop::IndexedDeclaration decl
   }
 
   if(FunctionDefinition* definition = dynamic_cast<FunctionDefinition*>(declaration))
-    declaration = definition->declaration(declaration->topContext());
+  {
+    // If this is a function-definition, and there is a separate declaration
+    // available, rename that declaration instead
+    Declaration* realDeclaration = definition->declaration(declaration->topContext());
+    if(realDeclaration)
+      declaration = realDeclaration;
+  }
 
   // if renaming a ctor, use the class instead which will trigger renaming of all ctors as well
   if(ClassFunctionDeclaration* cFunc = dynamic_cast<ClassFunctionDeclaration*>(declaration)) {
