@@ -815,23 +815,22 @@ void ExpressionVisitor::findMember( AST* node, AbstractType::Ptr base, const Ide
       expressionType( node, m_lastType, m_lastInstance );
   }
 
-  /** A postfix-expression is a primary expression together with a chain of sub-expressions that are applied from left to right
-   *
-   * have test */
+  /** A postfix-expression is a primary expression (or type-specifier)
+   * together with a chain of sub-expressions that are applied from left to right
+   */
 
   void ExpressionVisitor::visitPostfixExpression(PostfixExpressionAST* node)
   {
     clearLast();
-    if( node->type_specifier ) {
-      problem( node, "unexpected type-specifier" );
-      return;
-    }
-    if( !node->expression ) {
-      problem( node, "primary expression missing" );
-      return;
-    }
-    //First evaluate the primary expression, and then pass the result from sub-expression to sub-expression through m_lastType
-    visit( node->expression );
+    
+    //First evaluate the primary expression, or the type-specifier,
+    //and then pass the result from sub-expression to sub-expression through m_lastType
+    
+    if( node->type_specifier )
+      visit( node->type_specifier );
+
+    if( node->expression )
+      visit( node->expression );
 
     if( !node->sub_expressions )
       return;
@@ -2387,6 +2386,29 @@ void ExpressionVisitor::createDelayedType( AST* node , bool expression ) {
     }
   }
 
+  void ExpressionVisitor::visitLambdaExpression(LambdaExpressionAST *node)
+  {
+    DefaultVisitor::visitLambdaExpression(node);
+    FunctionType* type = new FunctionType;
+    if (node->declarator && node->declarator->parameter_declaration_clause) {
+      if (buildParametersFromDeclaration(node->declarator->parameter_declaration_clause)) {
+        foreach(const OverloadResolver::Parameter& param, m_parameters) {
+          type->addArgument(param.type);
+        }
+      }
+    }
+    if (node->declarator && node->declarator->trailing_return_type) {
+      visit(node->declarator->trailing_return_type);
+      type->setReturnType(m_lastType);
+    }
+    if (!type->returnType()) {
+      ///TODO: if body consists of only a single return statement, use that type as return type
+      type->setReturnType(AbstractType::Ptr(new IntegralType(IntegralType::TypeVoid)));
+    }
+    m_lastType = AbstractType::Ptr( type );
+    m_lastInstance = Instance(true);
+  }
+
   void ExpressionVisitor::visit(AST* node)
   {
     if (!node) {
