@@ -199,11 +199,15 @@ private slots:
   void testEnum2011_data();
   void testEnum2011();
   void testDecltype();
+  void testDecltypeUses();
   void testTrailingReturnType();
   void testConstexpr();
   void testInitListRegressions();
   void testBug284536();
   void testBug285004();
+  void testLambda();
+  void testLambdaReturn();
+  void testLambdaCapture();
   //END C++2011
 
 private:
