@@ -31,6 +31,7 @@
 #include <language/duchain/types/integraltype.h>
 #include <language/duchain/types/pointertype.h>
 #include <language/duchain/types/arraytype.h>
+#include <language/duchain/dumpchain.h>
 
 #include "typeutils.h"
 
@@ -260,6 +261,20 @@ void TestDUChain::testEnum2011()
   }
 }
 
+void TestDUChain::testDecltypeUses()
+{
+  QByteArray method("int c; decltype(c) c2 = decltype(c2)(c+5) + Test(q+5);");
+
+  LockedTopDUContext top = parse(method, DumpAll);
+
+  QCOMPARE(top->localDeclarations().size(), 2);
+  QVERIFY(top->localDeclarations()[0]->indexedType() == top->localDeclarations()[1]->indexedType());
+  QCOMPARE(top->localDeclarations()[0]->uses().size(), 1);
+  QCOMPARE(top->localDeclarations()[0]->uses().begin()->size(), 2);
+  QCOMPARE(top->localDeclarations()[1]->uses().size(), 1);
+  QCOMPARE(top->localDeclarations()[1]->uses().begin()->size(), 1);
+}
+
 void TestDUChain::testDecltype()
 {
   // see also: spec 7.1.6/4
@@ -484,3 +499,105 @@ void TestDUChain::testBug285004()
   QVERIFY(top);
   DUChainReadLocker lock;
 }
+
+void TestDUChain::testLambda()
+{
+  // see also: https://bugs.kde.org/show_bug.cgi?id=279699
+  const QByteArray code = "int main() {\n"
+                          "  int i;\n"
+                          "  auto f = [] (int i) { i = 0; };\n"
+                          "}\n";
+  LockedTopDUContext top = parse(code, DumpAll);
+  QVERIFY(top);
+  DUChainReadLocker lock;
+  QVERIFY(top->problems().isEmpty());
+  dumpDUContext(top);
+
+  DUContext* mainCtx = top->childContexts().last();
+  QCOMPARE(mainCtx->childContexts().size(), 2);
+  // (int i)
+  QCOMPARE(mainCtx->childContexts().first()->type(), DUContext::Function);
+  QCOMPARE(mainCtx->childContexts().first()->range(), RangeInRevision(2, 15, 2, 20));
+  // { i = 0; }
+  QCOMPARE(mainCtx->childContexts().last()->type(), DUContext::Other);
+  QCOMPARE(mainCtx->childContexts().last()->range(), RangeInRevision(2, 22, 2, 32));
+
+  // int i; in main context
+  QCOMPARE(mainCtx->localDeclarations().size(), 2);
+  Declaration* iDecl = mainCtx->localDeclarations().at(0);
+  // no uses
+  QCOMPARE(iDecl->uses().size(), 0);
+
+  // (int i) in lambda argument context
+  QCOMPARE(mainCtx->childContexts().first()->localDeclarations().size(), 1);
+  Declaration* iLambdaDecl = mainCtx->childContexts().first()->localDeclarations().first();
+  QCOMPARE(iLambdaDecl->uses().size(), 1);
+  QCOMPARE(iLambdaDecl->uses().begin()->size(), 1);
+  QCOMPARE(iLambdaDecl->uses().begin()->first(), RangeInRevision(2, 24, 2, 25));
+
+  Declaration* fDecl = mainCtx->localDeclarations().at(1);
+  TypePtr< FunctionType > funType = fDecl->type<FunctionType>();
+  QVERIFY(funType);
+  QCOMPARE(funType->indexedArgumentsSize(), 1u);
+  QVERIFY(funType->arguments().first()->equals(iLambdaDecl->abstractType().constData()));
+  QVERIFY(funType->returnType());
+  QVERIFY(funType->returnType().cast<IntegralType>());
+  QCOMPARE(funType->returnType().cast<IntegralType>()->dataType(), (uint) IntegralType::TypeVoid);
+}
+
+void TestDUChain::testLambdaReturn()
+{
+  // see also: https://bugs.kde.org/show_bug.cgi?id=279699
+  const QByteArray code = "int main() {\n"
+                          "  auto f = [] () -> int { return 1 };\n"
+                          "}\n";
+  LockedTopDUContext top = parse(code, DumpAll);
+  QVERIFY(top);
+  DUChainReadLocker lock;
+  QVERIFY(top->problems().isEmpty());
+  dumpDUContext(top);
+
+  DUContext* mainCtx = top->childContexts().last();
+  QCOMPARE(mainCtx->localDeclarations().size(), 1);
+  Declaration* fDecl = mainCtx->localDeclarations().at(0);
+  TypePtr< FunctionType > funType = fDecl->type<FunctionType>();
+  QVERIFY(funType->returnType());
+  QVERIFY(funType->returnType().cast<IntegralType>());
+  QCOMPARE(funType->returnType().cast<IntegralType>()->dataType(), (uint) IntegralType::TypeInt);
+}
+
+void TestDUChain::testLambdaCapture()
+{
+  // see also: https://bugs.kde.org/show_bug.cgi?id=279699
+  const QByteArray code = "int main() {\n"
+                          "  int i;\n"
+                          "  auto f = [&i] { i = 0; };\n"
+                          "}\n";
+  LockedTopDUContext top = parse(code, DumpAll);
+  QVERIFY(top);
+  DUChainReadLocker lock;
+  QVERIFY(top->problems().isEmpty());
+  dumpDUContext(top);
+
+  DUContext* mainCtx = top->childContexts().last();
+  QCOMPARE(mainCtx->childContexts().size(), 1);
+  // { i = 0; }
+  QCOMPARE(mainCtx->childContexts().last()->type(), DUContext::Other);
+  QCOMPARE(mainCtx->childContexts().last()->range(), RangeInRevision(2, 16, 2, 26));
+
+  // int i; in main context
+  QCOMPARE(mainCtx->localDeclarations().size(), 2);
+  Declaration* iDecl = mainCtx->localDeclarations().at(0);
+  QCOMPARE(iDecl->uses().size(), 1);
+  QCOMPARE(iDecl->uses().begin()->size(), 2);
+  QCOMPARE(iDecl->uses().begin()->first(), RangeInRevision(2, 13, 2, 14));
+  QCOMPARE(iDecl->uses().begin()->last(), RangeInRevision(2, 18, 2, 19));
+
+  Declaration* fDecl = mainCtx->localDeclarations().at(1);
+  TypePtr< FunctionType > funType = fDecl->type<FunctionType>();
+  QVERIFY(funType);
+  QCOMPARE(funType->indexedArgumentsSize(), 0u);
+  QVERIFY(funType->returnType());
+  QVERIFY(funType->returnType().cast<IntegralType>());
+  QCOMPARE(funType->returnType().cast<IntegralType>()->dataType(), (uint) IntegralType::TypeVoid);
+}
