@@ -28,10 +28,8 @@
 namespace TypeUtils {
 using namespace KDevelop;
 
-  AbstractType::Ptr realType(const AbstractType::Ptr& _base, const TopDUContext* /*topContext*/, bool* constant) {
+  AbstractType::Ptr realType(const AbstractType::Ptr& _base, const TopDUContext* /*topContext*/) {
 
-    if(constant)
-      *constant = false;
     AbstractType::Ptr base = _base;
     ReferenceType::Ptr ref = base.cast<ReferenceType>();
     TypeAliasType::Ptr alias = base.cast<TypeAliasType>();
@@ -97,8 +95,9 @@ using namespace KDevelop;
      * Ranks:
      * 1 - bool
      * 2 - 1 byte, char
+     * 2 - 2 byte, char16
      * 3 - 2 byte,  short int, wchar_t, unsigned short int
-     * 4 - 4 byte,  int, unsigned int
+     * 4 - 4 byte,  int, unsigned int, char32_t
      * 5 - 4 byte,  long int
      * 6 - 4 byte, long long int
      **/
@@ -107,11 +106,14 @@ using namespace KDevelop;
         return 1;
       break;
       case IntegralType::TypeChar:
+      case IntegralType::TypeChar16_t:
         return 2;
       break;
       case IntegralType::TypeWchar_t:
         return 3;
       break;
+      case IntegralType::TypeChar32_t:
+        return 4;
       case IntegralType::TypeInt:
         if( type->modifiers() & AbstractType::ShortModifier )
           return 3;
@@ -278,6 +280,21 @@ AbstractType::Ptr increasePointerDepth(AbstractType::Ptr type) {
     return newPointer.cast<AbstractType>();
 }
 
+IndexedType removeConstModifier(const IndexedType& indexedType)
+{
+    AbstractType::Ptr type = indexedType.abstractType();
+    removeConstModifier(type);
+    return type->indexed();
+}
+
+void removeConstModifier(AbstractType::Ptr& type)
+{
+  if(type && type->modifiers() & AbstractType::ConstModifier)
+  {
+    type->setModifiers(type->modifiers() & (~AbstractType::ConstModifier));
+  }
+}
+
 AbstractType::Ptr removeConstants(AbstractType::Ptr type, const TopDUContext* source) {
     ENSURE_CHAIN_READ_LOCKED
     if(TypePtr< EnumeratorType > enumerator = type.cast<EnumeratorType>())
