@@ -33,6 +33,7 @@ protected:
   virtual void visitBaseClause(BaseClauseAST *);
   virtual void visitBaseSpecifier(BaseSpecifierAST *);
   virtual void visitBinaryExpression(BinaryExpressionAST *);
+  virtual void visitBracedInitList(BracedInitListAST *);
   virtual void visitCastExpression(CastExpressionAST *);
   virtual void visitClassMemberAccess(ClassMemberAccessAST *);
   virtual void visitClassSpecifier(ClassSpecifierAST *);
@@ -52,6 +53,7 @@ protected:
   virtual void visitExpressionOrDeclarationStatement(ExpressionOrDeclarationStatementAST *);
   virtual void visitExpressionStatement(ExpressionStatementAST *);
   virtual void visitForStatement(ForStatementAST *);
+  virtual void visitForRangeDeclaration(ForRangeDeclarationAst* );
   virtual void visitFunctionCall(FunctionCallAST *);
   virtual void visitFunctionDefinition(FunctionDefinitionAST *);
   virtual void visitIfStatement(IfStatementAST *);
@@ -59,6 +61,7 @@ protected:
   virtual void visitInitDeclarator(InitDeclaratorAST *);
   virtual void visitInitializer(InitializerAST *);
   virtual void visitInitializerClause(InitializerClauseAST *);
+  virtual void visitInitializerList(InitializerListAST *);
   virtual void visitJumpStatement(JumpStatementAST *);
   virtual void visitLabeledStatement(LabeledStatementAST *);
   virtual void visitLinkageBody(LinkageBodyAST *);
@@ -105,6 +108,12 @@ protected:
   virtual void visitWinDeclSpec(WinDeclSpecAST *);
   virtual void visitSignalSlotExpression(SignalSlotExpressionAST *);
   virtual void visitQPropertyDeclaration(QPropertyDeclarationAST *);
+  virtual void visitTypeIDOperator(TypeIDOperatorAST *);
+  virtual void visitStaticAssert(StaticAssertAST *);
+  virtual void visitTrailingReturnType(TrailingReturnTypeAST *);
+  virtual void visitLambdaExpression(LambdaExpressionAST* );
+  virtual void visitLambdaCapture(LambdaCaptureAST* );
+  virtual void visitLambdaDeclarator(LambdaDeclaratorAST* );
 
 private:
   typedef void (Visitor::*visitor_fun_ptr)(AST *);
