@@ -102,43 +102,62 @@ char const * const names[] = {
   "UsingDirective",
   "WhileStatement",
   "WinDeclSpec",
-  "Kind_Comment",
-  "Kind_JumpStatement",
-  "Kind_SignalSlotExpression",
-  "Kind_QProperty"
+  "Comment",
+  "JumpStatement",
+  "SignalSlotExpression",
+  "QProperty",
+  "ForRangeDeclaration",
+  "TypeIDOperator",
+  "StaticAssert",
+  "TrailingReturnType",
+  "LambdaExpression",
+  "LambdaCapture",
+  "LambdaDeclarator",
+  "InitializerList",
+  "BracedInitList"
 };
 
 DumpTree::DumpTree()
-  : m_tokenStream(0), indent(0)
+  : m_tokenStream(0), indent(0), m_forceOutput(false)
 {
+  Q_ASSERT(sizeof(names) / sizeof(char const * const) == AST::NODE_KIND_COUNT);
 }
 
-void DumpTree::dump( AST * node, class TokenStream * tokenStream )
+void DumpTree::dump( AST * node, class TokenStream * tokenStream, bool forceOutput )
 {
   m_tokenStream = tokenStream;
+  m_forceOutput = forceOutput;
   visit(node);
   m_tokenStream = 0;
 }
 
+inline QDebug debug(bool forceOutput)
+{
+  return forceOutput ? qDebug() : kDebugStream(QtDebugMsg, 9007);
+}
+
 void DumpTree::visit(AST *node)
 {
+  if (!node) {
+    return;
+  }
+
   QString nodeText;
-  if( m_tokenStream && node ) {
-    for( std::size_t a = node->start_token; a != node->end_token; a++ ) {
+  if( m_tokenStream ) {
+    for( std::size_t a = node->start_token; a < node->end_token; a++ ) {
       const Token& tok( m_tokenStream->token((int) a) );
       nodeText += tok.symbolString() + ' ';
     }
   }
-  if (node)
-    kDebug(9007) << QString(indent * 2, ' ').toLatin1().constData() << names[node->kind]
-             <<  "[" << node->start_token << "," << node->end_token << "]" << nodeText << endl;
+
+  debug(m_forceOutput) << QString(indent * 2, ' ').toLatin1().constData() << names[node->kind]
+               <<  "[" << node->start_token << "," << node->end_token << "]" << nodeText;
 
   ++indent;
   DefaultVisitor::visit(node);
   --indent;
 
-  if (node)
-    kDebug(9007) << QString(indent * 2, ' ').toLatin1().constData() << names[node->kind];
+  debug(m_forceOutput) << QString(indent * 2, ' ').toLatin1().constData() << names[node->kind];
 }
 
 DumpTree::~ DumpTree( )
