@@ -200,6 +200,7 @@ KDevVarLengthArray<KDevVarLengthArray<QPair<uint, TOKEN_KIND>, 10 >, index_size
   ADD_TOKEN(extern);
   ADD_TOKEN(false);
   ADD_TOKEN(float);
+  ADD_TOKEN(final);
   ADD_TOKEN(for);
   ADD_TOKEN(friend);
   ADD_TOKEN(goto);
@@ -218,6 +219,7 @@ KDevVarLengthArray<KDevVarLengthArray<QPair<uint, TOKEN_KIND>, 10 >, index_size
   ADD_TOKEN(operator);
   ADD_TOKEN(or);
   ADD_TOKEN(or_eq);
+  ADD_TOKEN(override);
   ADD_TOKEN(private);
   ADD_TOKEN(protected);
   ADD_TOKEN(public);
