@@ -482,3 +482,61 @@ void TestParser::testInitListFalsePositives()
   QVERIFY(control.problems().isEmpty());
   }
 }
+
+void TestParser::memberVirtSpec()
+{
+  QFETCH(QString, data);
+  TranslationUnitAST* ast = parse( data.toLocal8Bit() );
+  QVERIFY(ast);
+  dump(ast);
+  QVERIFY(control.problems().isEmpty());
+}
+
+void TestParser::memberVirtSpec_data()
+{
+  QTest::addColumn<QString>("data");
+  QTest::newRow("override") <<
+    QString("struct A {\n"
+    "  virtual void f();\n"
+    "};\n"
+    "\n"
+    "struct B : A {\n"
+    "  void f() override;\n" // actual override token
+    "};\n"
+    "void foo() {\n"
+    "  int override = 0;\n" // identifier with special meaning
+    "}\n");
+  QTest::newRow("final") <<
+    QString("struct A {\n"
+    "  virtual void f() final;\n"
+    "};\n"
+    "void foo() {\n"
+    "  int final = 0;\n" // identifier with special meaning
+    "}\n");
+  QTest::newRow("new") <<
+    QString("struct A {\n"
+    "  virtual void f(int);\n"
+    "};\n"
+    "\n"
+    "struct B : A {\n"
+    "  void f() new;\n"
+    "};\n");
+}
+
+void TestParser::classVirtSpec()
+{
+  QFETCH(QString, data);
+  TranslationUnitAST* ast = parse( data.toLocal8Bit() );
+  QVERIFY(ast);
+  dump(ast);
+  QVERIFY(control.problems().isEmpty());
+}
+
+void TestParser::classVirtSpec_data()
+{
+  QTest::addColumn<QString>("data");
+  QTest::newRow("final") <<
+    QString("struct A final {};\n");
+  QTest::newRow("explicit") <<
+    QString("struct A explicit {};\n");
+}
