@@ -94,12 +94,10 @@ class KDEVCMAKECOMMON_EXPORT CMakeFolderItem
         QString buildDir() const { return m_buildDir; }
         void setBuildDir(const QString& bd) { m_buildDir = bd; }
         
-        void clear();
-
         KDevelop::ProjectTargetItem* targetNamed(Target::Type type, const QString& targetName) const;
         KDevelop::ProjectFolderItem* folderNamed(const QString& name) const;
-        void cleanupBuildFolders(const QList< Subdirectory >& subs);
-        void cleanupTargets(const QList<CMakeTarget>& targets);
+        QList<ProjectBaseItem*> cleanupBuildFolders(const QList<Subdirectory>& subs);
+        QList<ProjectBaseItem*> cleanupTargets(const QList<CMakeTarget>& targets);
     private:
         KDevelop::ReferencedTopDUContext m_topcontext;
         QStringList m_includeList;
