@@ -69,9 +69,11 @@ QString CMakeFunctionArgument::unescapeValue(const QString& value)
     return newValue;
 }
 
-void CMakeFunctionDesc::addArguments( const QStringList& args )
+void CMakeFunctionDesc::addArguments( const QStringList& args, bool addEvenIfEmpty )
 {
-    foreach( const QString& arg, args )
+    if(addEvenIfEmpty && args.isEmpty())
+        arguments += QString();
+    else foreach( const QString& arg, args )
     {
         CMakeFunctionArgument cmakeArg( arg );
         arguments.append( cmakeArg );
